-- Preferrably uses Monospace Fonts
local blinkrate = 0.5

function newTextInput(font, x, y, maxlength, align, padding, textcolor, boxcolor, bordercolor, text)
  local blinktimer = 0
  local blinkchar = '_'
  local align = align or 'left'
  local padding = padding or 2
  local wrap = font:getWidth('W') * maxlength -- W I think is the widest character, so I based it on that
  
  local txcolor = textcolor or {1,1,1}
  local bxcolor = boxcolor or {0,0,0,0.5}
  local bdcolor = bordercolor or {1,1,1}

  local textbox = {
    x = x, y = y,
    text = text or '',
  }
  
  function textbox.update(dt)
    if textbox.text:len() < maxlength then
      blinktimer = blinktimer + dt
      if blinktimer >= blinkrate then
        blinktimer = 0
      
        -- Inline Condition: var = <cond> and <var_true> or <var_false>
        blinkchar = blinkchar == '_' and ' ' or '_'
      end
    else
      -- Don't blink when maxlength is reached
      blinkchar = ''
    end
  end
  
  function textbox.draw()
    local h = font:getHeight('W')

    -- Filling the box
    love.graphics.setColor(bxcolor)
    love.graphics.rectangle('fill',
      x - padding,
      y - padding,
      wrap + (padding * 2),
      h + (padding * 2)
    )    

    -- Same thing for the border, except 'line'
    love.graphics.setColor(bdcolor)
    love.graphics.rectangle('line',
      x - padding,
      y - padding,
      wrap + (padding * 2),
      h + (padding * 2)
    )
    
    love.graphics.setFont(font)
    love.graphics.printf({txcolor, textbox.text .. blinkchar}, x, y, wrap, align)
  end
  
  function textbox.textinput(t)
    if textbox.text:len() < maxlength then
      textbox.text = textbox.text .. t
    end
  end
  
  function textbox.keypressed(key)
    if key == 'backspace' then
      textbox.text = textbox.text:sub(0, textbox.text:len() - 1)
    end
  end
  
  return textbox
end
