state_highscores = newState({100/255, 100/255, 100/255})

local title_font = love.graphics.newFont(70)
local title_text = 'High Scores'
local font = love.graphics.newFont(20)
local list = {}
local list_x = 150
local list_y = 80
local see_new = false

local menu = newMenu(love.graphics.newFont(35), 10)
menu.addButton('Back', love.graphics.getWidth() / 2, 550,
  function()
    local text = ''
    for i = 1, #list do
      list[i].status = 'old'
      text = text .. list[i].name .. ' ' .. list[i].score .. ' ' .. list[i].status .. '\n'
    end
    love.filesystem.write('highscores.lst', text)
    state_machine.change(state_title)
  end
)

local function sort_highscores()
  for champion = 1, #list - 1 do
    for challenger = champion + 1, #list do
      if list[champion].score < list[challenger].score then
        local name = list[champion].name
        local score = list[champion].score
        local status = list[champion].status
        
        list[champion].name = list[challenger].name
        list[champion].score = list[challenger].score
        list[champion].status = list[challenger].status
        
        list[challenger].name = name
        list[challenger].score = score
        list[challenger].status = status
      end
    end
  end
end

local function load_highscores()
  if #list == 0 then
    if not love.filesystem.getInfo('highscores.lst') then
      for i = 1, 20 do
        table.insert(list, {name = 'Inmate' .. love.math.random(10,99), score = love.math.random(200), status = 'old'})
      end
      sort_highscores()
    else
      for inmate in love.filesystem.lines('highscores.lst') do
        local info = {}
        for data in inmate:gmatch('%S+') do
          table.insert(info, data)
        end
      
        table.insert(list, {name = info[1], score = tonumber(info[2]), status = 'old'})
      end
    end  
  end
end

function state_highscores.load(params)
  see_new = params
  load_highscores()
end

function state_highscores.draw()
  love.graphics.setFont(title_font)
  love.graphics.print(title_text, love.graphics.getWidth() / 2 - title_font:getWidth(title_text) / 2, 10)
  
  love.graphics.setFont(font)
  for i, v in pairs(list) do
    if see_new and v.status == 'new' then
      love.graphics.print({{1,1,0}, v.name}, list_x, list_y + i * 20)
      love.graphics.printf({{1,1,0}, v.score}, love.graphics.getWidth() / 2, list_y + i * 20, love.graphics.getWidth() / 2 - list_x, 'right')
    else
      love.graphics.print(v.name, list_x, list_y + i * 20)
      love.graphics.printf(v.score, love.graphics.getWidth() / 2, list_y + i * 20, love.graphics.getWidth() / 2 - list_x, 'right')
    end
  end
  menu.draw()
end

function state_highscores.isObtained(score)
  local evaluation = false
  load_highscores()
  for _, v in pairs(list) do
    if v.score < score then
      evaluation = true
    end
  end
  
  return evaluation
end
function state_highscores.enter(name, score)
  list[#list].name = name
  list[#list].score = score
  list[#list].status = 'new'
  sort_highscores()
end

function state_highscores.keypressed(key)
  if key == 'return' or key == 'space' then
    menu.button_press()
  else
    menu.cycle_down()
  end
end
function state_highscores.keyreleased(key)
  if key == 'return' or key == 'space' then
    menu.button_release()
  end
end

function state_highscores.mousemoved(x, y, dx, dy)
  menu.mouse_over(x, y)
end
function state_highscores.mousepressed(x, y, button)
  if button == 1 then menu.button_press() end
end
function state_highscores.mousereleased(x, y, button)
  if button == 1 then menu.button_release() end
end
