walls = {
  size = 10,
}

lasers = {}

local think = 0
local thinktime = 4
local firerate = 1
local boxcolor = {0, 100/255, 0}
local num = 0
local font = love.graphics.newFont(20)

local message = ''
local message_a = 1
local message_y = 10
local messages = {
  'Easy, huh?',
  'Any second now...',
  "It's stuffy in there.",
  'Having fun in the only level?',
  'Frantically standing still?',
  "Let's play forever!",
  "Esc means Pause, because you can't escape.",
  "Pressing only 4 keys isn't stressful",
  'Almost had you. :(',
  "Who's managing those lasers!? ;)"
}

local charge = love.audio.newSource('sfx/charge.wav', 'static')
local shoot = love.audio.newSource('sfx/shoot.mp3', 'static')
local rushtimer = 5000

local function fireUp(i)
  shoot:stop()
  shoot:play()
  table.insert(lasers, {x = walls.up[i].x, y = walls.up[i].y + walls.size, w = walls.size, h = #walls.left * walls.size, life = 1})
end

local function fireDown(i)
  shoot:stop()
  shoot:play()
  table.insert(lasers, {x = walls.up[i].x, y = walls.up[i].y + walls.size, w = walls.size, h = #walls.left * walls.size, life = 1})
end

local function fireLeft(i)
  shoot:stop()
  shoot:play()
  table.insert(lasers, {x = walls.left[i].x + walls.size, y = walls.left[i].y, w = #walls.up * walls.size, h = walls.size, life = 1})
end

local function fireRight(i)
  shoot:stop()
  shoot:play()
  table.insert(lasers, {x = walls.left[i].x + walls.size, y = walls.left[i].y, w = #walls.up * walls.size, h = walls.size, life = 1})
end

function walls.initialize(n, msg)
  walls.up = {}
  walls.down = {}
  walls.left = {}
  walls.right = {}
  lasers = {}
  thinktime = 3
  think = 0
  rushtimer = 5
  message = msg
  message_a = 1
  message_y = 30
  num = n > 10 and n or 10
  local x = (love.graphics.getWidth() / 2) - ((num * walls.size) / 2)
  local y = (love.graphics.getHeight() / 2) - ((num * walls.size) / 2)
  
  for i = 1, num - 1 do
    table.insert(walls.up, {x = x + (i * walls.size), y = y, firing = false, firetime = firerate})
    table.insert(walls.down, {x = x + (i * walls.size), y = y + (walls.size * num), firing = false, firetime = firerate})
    
    table.insert(walls.left, {x = x, y = y + (i * walls.size), firing = false, firetime = firerate})
    table.insert(walls.right, {x = x + (walls.size * num), y = y + (i * walls.size), firing = false, firetime = firerate})
  end
end

function walls.update(dt)
  message_a = message_a > 0 and message_a - (dt/3) or 0
  message_y = message_y > -10 and message_y - 10 * dt or  -10
  think = think + dt
  if think >= thinktime then
    think = 0
    thinktime = thinktime > 0.4 and thinktime - 0.2 or 0.4
    local rand_wall = love.math.random(4)
    if rand_wall == 1 then
      walls.up[love.math.random(#walls.up)].firing = true
      charge:stop()
      charge:play()
    elseif rand_wall == 2 then
      walls.down[love.math.random(#walls.down)].firing = true
      charge:stop()
      charge:play()
    elseif rand_wall == 3 then
      walls.left[love.math.random(#walls.left)].firing = true
      charge:stop()
      charge:play()
    else
      walls.right[love.math.random(#walls.right)].firing = true
      charge:stop()
      charge:play()
    end
  end

  if thinktime == 0.4 then
    rushtimer = rushtimer - dt
    if rushtimer <= 0 then
      walls.initialize(num-2, messages[love.math.random(#messages)])
    end
  end
  
  for i, v in pairs(walls.up) do
    if v.firing then
      v.firetime = v.firetime - dt
      if v.firetime <= 0 then
        v.firing = false
        v.firetime = firerate
        fireUp(i)
      end
    end
  end
  
  for i, v in pairs(walls.down) do
    if v.firing then
      v.firetime = v.firetime - dt
      if v.firetime <= 0 then
        v.firing = false
        v.firetime = firerate
        fireDown(i)
      end
    end
  end
  
  for i, v in pairs(walls.left) do
    if v.firing then
      v.firetime = v.firetime - dt
      if v.firetime <= 0 then
        v.firing = false
        v.firetime = firerate
        fireLeft(i)
      end
    end
  end
  
  for i, v in pairs(walls.right) do
    if v.firing then
      v.firetime = v.firetime - dt
      if v.firetime <= 0 then
        v.firing = false
        v.firetime = firerate
        fireRight(i)
      end
    end
  end
  
  local laser_killer = {}
  
  for i, v in ipairs(lasers) do
    v.life = v.life - dt
    if v.life <= 0 then
      table.insert(laser_killer, i)
    end
  end
  
  for i = #laser_killer, 1, -1 do
    table.remove(lasers, laser_killer[i])
  end
  
end

function walls.draw()
  love.graphics.setFont(font)
  love.graphics.setColor(1,1,1, message_a)
  love.graphics.printf(message, 0, message_y, love.graphics.getWidth(), 'center')
  for _, v in pairs(walls.up) do
    love.graphics.setColor(boxcolor)
    if v.firing then
      love.graphics.setColor(1, (v.firetime * 255)/255, 0)
    end
    love.graphics.rectangle('fill', v.x, v.y, walls.size, walls.size)
  end
  
  for _, v in pairs(walls.down) do
    love.graphics.setColor(boxcolor)
    if v.firing then
      love.graphics.setColor(1, (v.firetime * 255)/255, 0)
    end
    love.graphics.rectangle('fill', v.x, v.y, walls.size, walls.size)
  end
  
  for _, v in pairs(walls.left) do
    love.graphics.setColor(boxcolor)
    if v.firing then
      love.graphics.setColor(1, (v.firetime * 255)/255, 0)
    end
    love.graphics.rectangle('fill', v.x, v.y, walls.size, walls.size)
  end
  
  for _, v in pairs(walls.right) do
    love.graphics.setColor(boxcolor)
    if v.firing then
      love.graphics.setColor(1, (v.firetime * 255)/255, 0)
    end
    love.graphics.rectangle('fill', v.x, v.y, walls.size, walls.size)
  end
  
  for _, v in pairs(lasers) do
    love.graphics.setColor(1, 0, 0, v.life)
    love.graphics.rectangle('fill', v.x, v.y, v.w, v.h)
  end
end
